<?php

namespace Drupal\commerce_checkout_link\Controller;

use Drupal\commerce_cart\CartSessionInterface;
use Drupal\commerce_checkout\CheckoutOrderManagerInterface;
use Drupal\commerce_checkout_link\CheckoutLinkManager;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\OrderAssignmentInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Returns responses for Commerce Checkout Link routes.
 */
class CommerceCheckoutLinkController extends ControllerBase {

  /**
   * Time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Cart session.
   *
   * @var \Drupal\commerce_cart\CartSessionInterface
   */
  protected $cartSession;

  /**
   * Order assignment.
   *
   * @var \Drupal\commerce_order\OrderAssignmentInterface
   */
  protected $orderAssignment;

  /**
   * Checkout order manager.
   *
   * @var \Drupal\commerce_checkout\CheckoutOrderManagerInterface
   */
  protected $checkoutOrderManager;

  /**
   * CommerceCheckoutLinkController constructor.
   */
  public function __construct(TimeInterface $time, AccountProxyInterface $current_user, CartSessionInterface $cart_session, OrderAssignmentInterface $orderAssignment, EntityTypeManagerInterface $entityTypeManager, CheckoutOrderManagerInterface $checkoutOrderManager) {
    $this->time = $time;
    $this->currentUser = $current_user;
    $this->cartSession = $cart_session;
    $this->orderAssignment = $orderAssignment;
    $this->entityTypeManager = $entityTypeManager;
    $this->checkoutOrderManager = $checkoutOrderManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('datetime.time'),
      $container->get('current_user'),
      $container->get('commerce_cart.cart_session'),
      $container->get('commerce_order.order_assignment'),
      $container->get('entity_type.manager'),
      $container->get('commerce_checkout.checkout_order_manager')
    );
  }

  /**
   * Builds the response.
   */
  public function checkout(OrderInterface $commerce_order, $timestamp, $hash) {
    $current_time = $this->time->getCurrentTime();
    $timeout = 24 * 3600;
    if ($current_time - $timestamp > $timeout) {
      throw new AccessDeniedHttpException('The link provided is no longer valid');
    }
    // If we do not have a hash, we do not allow this.
    if (!$hash) {
      throw new AccessDeniedHttpException('The hash provided was not valid');
    }
    // If the order has changed after this timestamp, let's not allow it.
    if ($timestamp < $commerce_order->getChangedTime()) {
      throw new AccessDeniedHttpException('The order was changed after the link was generated');
    }
    // If the hash does not look like we want, let's not allow it.
    if (!hash_equals($hash, CheckoutLinkManager::generateHash($timestamp, $commerce_order))) {
      throw new AccessDeniedHttpException('The hash provided did not match the expected hash');
    }
    // Ok, we'll allow it. First reassign the order.
    $this->orderAssignment->assign($commerce_order, $this->entityTypeManager->getStorage('user')->load($this->currentUser->id()));
    if ($this->currentUser->isAnonymous()) {
      $this->cartSession->addCartId($commerce_order->id());
    }
    return $this->redirect('commerce_checkout.form', [
      'commerce_order' => $commerce_order->id(),
      'step' => $this->checkoutOrderManager->getCheckoutStepId($commerce_order),
    ]);
  }

}
